@@ -5,7 +5,7 @@
 // Copyright (c) 2001 Mike Kestner
 //
 // This program is free software; you can redistribute it and/or
-// modify it under the terms of version 2 of the Lesser GNU General
+// modify it under the terms of version 2 of the Lesser GNU General 
 // Public License as published by the Free Software Foundation.
 //
 // This program is distributed in the hope that it will be useful,
@@ -23,17 +23,10 @@ namespace Gtk {
 	using System;
 	using System.Reflection;
 	using System.Runtime.InteropServices;
+	using System.Threading;
 	using Gdk;
 
-	public class Application {
-		static System.Threading.Thread MainThread;
-
-		//
-		// Disables creation of instances.
-		//
-		private Application ()
-		{
-		}
+	public partial class Application {
 
 		const int WS_EX_TOOLWINDOW = 0x00000080;
 		const int WS_OVERLAPPEDWINDOW = 0x00CF0000;
@@ -42,7 +35,7 @@ namespace Gtk {
 		static extern IntPtr Win32CreateWindow (int dwExStyle, string lpClassName, string lpWindowName,int dwStyle, int x, int y, int nWidth, int nHeight, IntPtr hWndParent, IntPtr hMenu, IntPtr hInstance, IntPtr lParam);
 
 		[DllImport ("user32.dll", EntryPoint="DestroyWindow", CharSet=CharSet.Unicode, CallingConvention=CallingConvention.StdCall)]
-		static extern bool Win32DestroyWindow (IntPtr window);
+		static extern bool Win32DestroyWindow (IntPtr window); 
 
 		static Application ()
 		{
@@ -63,15 +56,17 @@ namespace Gtk {
 			}
 		}
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern void gtk_init (ref int argc, ref IntPtr argv);
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern bool gtk_init_check (ref int argc, ref IntPtr argv);
 
 		static void SetPrgname ()
 		{
-			GLib.Global.ProgramName = System.IO.Path.GetFileNameWithoutExtension (Environment.GetCommandLineArgs () [0]);
+			var args = Environment.GetCommandLineArgs ();
+			if (args != null && args.Length > 0)
+				GLib.Global.ProgramName = System.IO.Path.GetFileNameWithoutExtension (args [0]);
 		}
 
 		public static void Init ()
@@ -81,6 +76,8 @@ namespace Gtk {
 			int argc = 0;
 
 			gtk_init (ref argc, ref argv);
+
+			SynchronizationContext.SetSynchronizationContext (new GLib.GLibSynchronizationContext ());
 		}
 
 		static bool do_init (string progname, ref string[] args, bool check)
@@ -118,26 +115,17 @@ namespace Gtk {
 			return res;
 		}
 
-		internal static void AssertMainThread ()
-		{
-			if (MainThread != null && System.Threading.Thread.CurrentThread != MainThread) {
-				GLib.Log.Write (null, GLib.LogLevelFlags.Warning, "Gtk operations should be done on the main Thread\n" + Environment.StackTrace);
-			}
-		}
-
 		public static void Init (string progname, ref string[] args)
 		{
-			MainThread = System.Threading.Thread.CurrentThread;
 			do_init (progname, ref args, false);
 		}
 
 		public static bool InitCheck (string progname, ref string[] args)
 		{
-			MainThread = System.Threading.Thread.CurrentThread;
 			return do_init (progname, ref args, true);
 		}
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern void gtk_main ();
 
 		public static void Run ()
@@ -145,18 +133,19 @@ namespace Gtk {
 			gtk_main ();
 		}
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern bool gtk_events_pending ();
 
+
 		public static bool EventsPending ()
 		{
 			return gtk_events_pending ();
 		}
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern void gtk_main_iteration ();
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern bool gtk_main_iteration_do (bool blocking);
 
 		public static void RunIteration ()
@@ -168,8 +157,8 @@ namespace Gtk {
 		{
 			return gtk_main_iteration_do (blocking);
 		}
-
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern void gtk_main_quit ();
 
 		public static void Quit ()
@@ -178,14 +167,12 @@ namespace Gtk {
 		}
 
 
-		[DllImport("libgtk-win32-2.0-0.dll", CallingConvention=CallingConvention.Cdecl)]
+		[DllImport (Global.GtkNativeDll, CallingConvention = CallingConvention.Cdecl)]
 		static extern IntPtr gtk_get_current_event ();
 
 		public static Gdk.Event CurrentEvent {
 			get {
-				var raw_ret = gtk_get_current_event ();
-				var ret = Gdk.Event.GetEvent (raw_ret, true);
-				return ret;
+				return Gdk.Event.GetEvent (gtk_get_current_event ());
 			}
 		}
 
@@ -193,39 +180,39 @@ namespace Gtk {
 			EventHandler d;
 			object sender;
 			EventArgs args;
-
+			
 			internal InvokeCB (EventHandler d)
 			{
 				this.d = d;
 				args = EventArgs.Empty;
 				sender = this;
 			}
-
+			
 			internal InvokeCB (EventHandler d, object sender, EventArgs args)
 			{
 				this.d = d;
 				this.args = args;
 				this.sender = sender;
 			}
-
+			
 			internal bool Invoke ()
 			{
 				d (sender, args);
 				return false;
 			}
 		}
-
+		
 		public static void Invoke (EventHandler d)
 		{
 			InvokeCB icb = new InvokeCB (d);
-
+			
 			GLib.Timeout.Add (0, new GLib.TimeoutHandler (icb.Invoke));
 		}
 
 		public static void Invoke (object sender, EventArgs args, EventHandler d)
 		{
 			InvokeCB icb = new InvokeCB (d, sender, args);
-
+			
 			GLib.Timeout.Add (0, new GLib.TimeoutHandler (icb.Invoke));
 		}
 	}
